
// In place array reversal

predicate easyReverse(s : seq<int>, t : seq<int>)
{
    |s| == |t| && forall i: int :: 0 <= i < |s|  ==> s[i] == t[|t| - i - 1]
}


method Reverse(a: array<int>, left: nat, right: nat) 
   modifies a
   requires 0 <= left <= right < a.Length 
   ensures easyReverse(a[left..right+1], old(a[left..right+1])) // prove this for part (a)
   // ensures ?  -> add your postcondition to solve part (b)
{
   if a.Length <= 1 { return; }

   var l, r := left, right;
 
    while l < r
    {

        a[l], a[r] := a[r], a[l]; 
        r := r - 1;
        l := l + 1;
    }
}


// Swapping the two partitions of array divided by index (the first partition is inclusive of a[index]) in-place
// without using any auxiliary memory, but relying on the in-place reversal method implemented above
method ArraySwap (a: array<int>, index: nat) 
   modifies a
   requires a.Length > 0
   requires 0 <= index < a.Length
   ensures a[..] == old(a[index+1..]) + old(a[..index+1]) // prove this for part (c)
{
   if a.Length <= 1 || index == a.Length - 1 {return; }

   Reverse(a, 0, index);
   Reverse(a, index+1, a.Length - 1);
   Reverse(a, 0, a.Length-1);
}